==> spoofer-0.7.3.tar.gz: spoofer-0.7.3/messages.c <==
/****************************************************************************
   Program:     $Id: messages.c 541 2009-03-27 20:36:43Z rbeverly $
   Author:      Rob Beverly <rbeverly at csail.mit.edu>
   Date:        $Date: 2009-03-27 16:36:43 -0400 (Fri, 27 Mar 2009) $
   Description: Spoofer control channel message routines
****************************************************************************/
#include "spoof.h"

/******************** message printing/debug ********************/

char *type(uint8_t typ) {
    static char msg_type_name[SMALLBUF];
    switch(typ){
        case DEFAULT:
            strncpy(msg_type_name, "DEFAULT", sizeof(msg_type_name)); 
            break;
        case HELLO:
            strncpy(msg_type_name, "HELLO", sizeof(msg_type_name)); 
            break;
        case PROBE_REQ:
            strncpy(msg_type_name, "PROBE_REQ", sizeof(msg_type_name)); 
            break;
        case REPORT:
            strncpy(msg_type_name, "REPORT", sizeof(msg_type_name)); 
            break;
        case TRACE:
            strncpy(msg_type_name, "TRACE", sizeof(msg_type_name)); 
            break;
        case TRACEFILT:
            strncpy(msg_type_name, "TRACEFILT", sizeof(msg_type_name)); 
            break;
        case ERR:
            strncpy(msg_type_name, "ERR", sizeof(msg_type_name)); 
            break;
        case DONE:
            strncpy(msg_type_name, "DONE", sizeof(msg_type_name)); 
            break;
        default:
            strncpy(msg_type_name, "INVALID", sizeof(msg_type_name)); 
            break;
    }
    return (msg_type_name);
}

void printAddressList(struct in_addr *iplist, int num) {
    int i;
    for (i=0;i<num;i++) {
        if (&iplist[i] == NULL) return;
        printf("\tIP: %s\n", inet_ntoa(iplist[i]));
    }
}

void printTest(test_t *test, int detail) {
    static char buf[SMALLBUF];
    struct in_addr a;
    time_t ts;

    ts = ntohl(test->ts);
    printf(">> Probe [ts: %s] ", ctime(&ts));
    snprintf(buf, SEQNOSIZE, "%s", test->seqno);
    a.s_addr = test->src_addr;
    printf("Src: %s ", inet_ntoa(a));
    a.s_addr = test->dst_addr;
    printf("Dst: %s ", inet_ntoa(a));
    if (detail) {
        printf("Seq: %s HMAC: ", buf);
        binDump(test->hmac, HMACSIZE);
    }
}

void printTestList(unsigned char *msg, int verbose) {
    addr_list_t *list = (addr_list_t *) msg; 
    test_t *test;
    int i;

    if (ntohs(list->size) > 100) {
        warn("bogus test list.");
        return;
    }
    if (verbose) {
        test = (test_t *) (msg + sizeof(addr_list_t));
        for (i=0;i<ntohs(list->size);i++) {
            printTest(test, TRUE);
            test++;
        }
    }
}

char *printCharStr(unsigned char *str, int len) {
    static char buf[BIGBUF];
    memset(buf, 0, sizeof(buf));
    snprintf(buf, len+1, "%s", str);
    return (buf);
}

void printTrace(unsigned char *str, int len) {
    struct in_addr dst;
    memcpy(&dst, str, sizeof(struct in_addr));
    len -= sizeof(struct in_addr);
    printf("Trace Destination: %s\n", inet_ntoa(dst));
    printf("%s\n", printCharStr(str + sizeof(struct in_addr), len));
}

void printHelloMsg(unsigned char *msg, int verbose) {
    hello_msg_t *hello = (hello_msg_t *) msg; 
    unsigned char *os = msg + sizeof(hello_msg_t);
    struct in_addr addr;
    addr.s_addr = hello->ip;
    printf(">> HELLO: [Client: %s][Dist: %d][OSlen: %d]",
            inet_ntoa(addr), hello->dist, hello->oslen);
    if (hello->oslen >0) 
        printf("[OS: %s]", printCharStr(os, hello->oslen));
    printf("[SID: %lu]\n", (unsigned long) ntohl(hello->sessionid));
}

void printReportMsg(unsigned char *msg) {
    report_msg_t *report = (report_msg_t *) msg;
    unsigned char *seq = msg + sizeof(report_msg_t);
    struct in_addr addr;
    printf("Report Message:\n");
    addr.s_addr = report->src_ip;
    printf("\tSrc IP: %s\n", inet_ntoa(addr));
    addr.s_addr = report->dst_ip;
    printf("\tDst IP: %s\n", inet_ntoa(addr));
    if (report->seqlen > 0) {
        printf("\tSeq(%d): ", report->seqlen);
        printf("%s\n", printCharStr(seq, report->seqlen));
    }
}

void printTraceFilter(unsigned char *msg) {
    addr_list_t *list = (addr_list_t *) msg;
    tracefilter_test_t *test = (tracefilter_test_t *) (msg + sizeof(addr_list_t));
    struct in_addr addr;
    int i;

    if (ntohs(list->size) > 100) {
        warn("bogus tracefilter list");
        return;
    }
    printf("TraceFilter (Targets: %d)\n", ntohs(list->size));
    for (i=0;i<ntohs(list->size);i++) {
        printf("\tPathLen: %d\n", test->dist);
        addr.s_addr = test->dst_ip;
        printf("\tDst IP: %s\n", inet_ntoa(addr));
        test++;
    }
}

void printMsg(unsigned char *msg, int verbose) {
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    unsigned char *payload;
    int len = ntohs(hdr->len);
    if (!verbose) {
        printf(">> %s Msg v%d: [Flag: %d][Ack: %d][Len: %d]\n",
            type(hdr->type), hdr->ver, hdr->flag, hdr->ack, len);
    } else {
        printf("Message Header:\n");
        printf("\tVer:  %d\n", hdr->ver);
        printf("\tType: %s\n", type(hdr->type));
        printf("\tFlag: %d\n", hdr->flag);
        printf("\tAck:  %d\n", hdr->ack);
        printf("\tLen:  %d\n", len);
    }
    if (len == 0) return;
    payload = msg + sizeof(spoof_msg_t);
    switch(hdr->type){
        case HELLO:
            printHelloMsg(payload, verbose);
            break;
        case PROBE_REQ:
            printTestList(payload, verbose);
            break;
        case REPORT:
            printReportMsg(payload);
            break;
        case TRACE:
            printTrace(payload, len);
            break;
        case TRACEFILT:
            printTraceFilter(payload);
            break;
        case ERR:
            printCharStr(payload, len);
            break;
        case DONE:
            if (verbose) printf("%s\n", printCharStr(payload, len));
            break;
        default:
            printf("*** Error, invalid message payload\n");
    }
}


/******************** message kraftwerk ********************/

int readMsg(int sock, spoof_msg_t **message, enum msg_types type) {
    unsigned char *buf = (unsigned char *) *message; 
    spoof_msg_t *msg = *message;
    
    memset(buf, 0, BIGBUF); 
    int n = Readn(sock, buf, sizeof(spoof_msg_t), CLIENT_TIMEOUT);
    if (n != sizeof(spoof_msg_t)) {
        warn("Timeout (%d) reading spoof_msg_t from socket, got %d bytes",
            CLIENT_TIMEOUT, n);
        return -1;
    }
    msg = (spoof_msg_t *) buf;
    if (ntohs(msg->len) > MAXMSGSIZE)
        return 0;
    if (ntohs(msg->len) > 0) {
        if ((n+= Readn(sock, buf + sizeof(spoof_msg_t), ntohs(msg->len), CLIENT_TIMEOUT)) <= 0)
            return 0;   /* connection closed by other end */
        if (n != sizeof(spoof_msg_t) + ntohs(msg->len)) {
            warn("Timeout (%d) reading spoof msg body from socket, got %d bytes",
                CLIENT_TIMEOUT, n);
            return -1;
        }
#if DEBUG 
        printMsg(buf, VERBOSE);
#endif
    }
    if (msg->type != type) {
        warn("Bad message type (%d expected %d)", msg->type, type);
        return -1;
    }
    return(n);
}

void defaultMsg(spoof_msg_t *msg) {
    if (msg) {
        memset(msg, 0, sizeof(spoof_msg_t));
        msg->ver= VERSION;
        msg->type = DEFAULT;
        msg->flag = 0;
        msg->ack = 0;
        msg->len = 0;
        msg->unused = 0;
    }
}

int errorRequest(unsigned char *msg) {
    int msglen = sizeof(spoof_msg_t);
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    
    defaultMsg(hdr);
    hdr->type = ERR;
    return(msglen);
}

int helloRequest(unsigned char *msg) {
    int oslen = strlen(OS);
    int msglen = sizeof(spoof_msg_t) + sizeof(hello_msg_t) + oslen;
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    hello_msg_t *hello = (hello_msg_t *) (msg + sizeof(spoof_msg_t));
    unsigned char *os = msg + sizeof(spoof_msg_t) + sizeof(hello_msg_t);
    
    defaultMsg(hdr);
    hdr->type = HELLO;
    hdr->len = htons(sizeof(hello_msg_t) + oslen);
    hello->ip = 0;
    hello->dist = 0;
    hello->unused = 0;
    hello->oslen = oslen;
    memcpy(os, OS, oslen);
    hello->sessionid = 0;
    return(msglen);
}

int reportRequest(unsigned char *msg, unsigned char *seqno, 
        unsigned long src_addr, unsigned long dst_addr, int nat) {
    int msglen = sizeof(spoof_msg_t) + sizeof(report_msg_t) + SEQNOSIZE;
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    report_msg_t *report = (report_msg_t *) (msg + sizeof(spoof_msg_t));
    unsigned char *seq = msg + sizeof(spoof_msg_t) + sizeof(report_msg_t);
    
    defaultMsg(hdr);
    hdr->type = REPORT;
    hdr->len = htons(sizeof(report_msg_t) + SEQNOSIZE);
    report->src_ip = src_addr; 
    report->dst_ip = dst_addr; 
    report->seqlen = SEQNOSIZE;
    report->ack = 1;
    if (strncmp((char *)seqno, "SENDTOFAIL", strlen("SENDTOFAIL")) == 0) {
        report->ack = 0;
    }
    /* locally detected NAT interface in use; let server know */
    if (nat) {
        report->ack = 2;
    }
    memcpy(seq, seqno, SEQNOSIZE);
    return(msglen);
}

int doneRequest(unsigned char *msg) {
    int msglen = sizeof(spoof_msg_t);
    spoof_msg_t *hdr = (spoof_msg_t *) msg;

    defaultMsg(hdr);
    hdr->type = DONE;
    return(msglen);
}

int traceFilterRequest(unsigned char *msg) {
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    defaultMsg(hdr);
    hdr->type = TRACEFILT;
    return (sizeof(spoof_msg_t));
}

int probeRequest(unsigned char *msg) {
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    defaultMsg(hdr);
    hdr->type = PROBE_REQ;
    return (sizeof(spoof_msg_t));
}

int readProbeList(unsigned char *buf, test_t **testlistptr) {
    test_t *testlist;
    addr_list_t *addr = (addr_list_t *) (buf + sizeof(spoof_msg_t));
    int numAddr = ntohs(addr->size);
    int i;

    if ((*testlistptr = calloc(numAddr, sizeof(test_t))) == NULL)
        fatal("calloc error in %s\n", __func__);
    testlist = *testlistptr;
    for (i=0;i<numAddr;i++) {
        memcpy(testlist + i, 
            buf + sizeof(spoof_msg_t) + sizeof(addr_list_t) + (sizeof(test_t)*i), 
            sizeof(test_t));
    }
    return(numAddr);
}

==> spoofer-0.8.tar.gz: spoofer-0.8/messages.c <==
/****************************************************************************
   Program:     $Id: messages.c 617 2009-05-08 17:57:37Z rbeverly $
   Author:      Rob Beverly <rbeverly at csail.mit.edu>
   Date:        $Date: 2009-05-08 13:57:37 -0400 (Fri, 08 May 2009) $
   Description: Spoofer control channel message routines
****************************************************************************/
#include "spoof.h"

/******************** message printing/debug ********************/

char *type(uint8_t typ) {
    static char msg_type_name[SMALLBUF];
    switch(typ){
        case DEFAULT:
            strncpy(msg_type_name, "DEFAULT", sizeof(msg_type_name)); 
            break;
        case HELLO:
            strncpy(msg_type_name, "HELLO", sizeof(msg_type_name)); 
            break;
        case PROBE_REQ:
            strncpy(msg_type_name, "PROBE_REQ", sizeof(msg_type_name)); 
            break;
        case PROBE6_REQ:
            strncpy(msg_type_name, "PROBE6_REQ", sizeof(msg_type_name)); 
            break;
        case REPORT:
            strncpy(msg_type_name, "REPORT", sizeof(msg_type_name)); 
            break;
        case REPORT6:
            strncpy(msg_type_name, "REPORT6", sizeof(msg_type_name)); 
            break;
        case TRACE:
            strncpy(msg_type_name, "TRACE", sizeof(msg_type_name)); 
            break;
        case TRACEFILT:
            strncpy(msg_type_name, "TRACEFILT", sizeof(msg_type_name)); 
            break;
        case ERR:
            strncpy(msg_type_name, "ERR", sizeof(msg_type_name)); 
            break;
        case DONE:
            strncpy(msg_type_name, "DONE", sizeof(msg_type_name)); 
            break;
        default:
            strncpy(msg_type_name, "INVALID", sizeof(msg_type_name)); 
            break;
    }
    return (msg_type_name);
}

void printAddressList(struct in_addr *iplist, int num) {
    int i;
    for (i=0;i<num;i++) {
        if (&iplist[i] == NULL) return;
        printf("\tIP: %s\n", inet_ntoa(iplist[i]));
    }
}

void printTest(test_t *test, int detail) {
    static char buf[SMALLBUF];
    struct in_addr a;
    struct in6_addr a6;
    time_t ts;

    ts = ntohl(test->ts);
    snprintf(buf, SMALLBUF, "%s", ctime(&ts));
    buf[strlen(buf)-1] = '\0';
    printf(">> Probe [ts: %s] ", buf);
    if (isIPv4(test->src_addr)) {
        a.s_addr = n6ton4(test->src_addr);
        printf("Src: %s ", inet_ntoa(a));
    } else {
        memcpy(a6.s6_addr, test->src_addr, IPV6ADDRLEN);
        printf("Src: %s ", inet_ntop(AF_INET6, &a6, buf, SMALLBUF));
    }
    if (isIPv4(test->dst_addr)) {
        a.s_addr = n6ton4(test->dst_addr);
        printf("Dst: %s ", inet_ntoa(a));
    } else {
        memcpy(a6.s6_addr, test->dst_addr, IPV6ADDRLEN);
        printf("Dst: %s ", inet_ntop(AF_INET6, &a6, buf, SMALLBUF));
    }
    snprintf(buf, SEQNOSIZE, "%s", test->seqno);
    if (detail) {
        printf("Seq: %s HMAC: ", buf);
        binDump(test->hmac, HMACSIZE);
    }
}

void printTestList(unsigned char *msg, int verbose) {
    addr_list_t *list = (addr_list_t *) msg; 
    test_t *test;
    int i;

    if (ntohs(list->size) > 100) {
        warn("bogus test list.");
        return;
    }
    if (verbose) {
        test = (test_t *) (msg + sizeof(addr_list_t));
        for (i=0;i<ntohs(list->size);i++) {
            printTest(test, TRUE);
            test++;
        }
    }
}

char *printCharStr(unsigned char *str, int len) {
    static char buf[BIGBUF];
    memset(buf, 0, sizeof(buf));
    snprintf(buf, len+1, "%s", str);
    return (buf);
}

void printTrace(unsigned char *str, int len) {
    struct in_addr dst;
    memcpy(&dst, str, sizeof(struct in_addr));
    len -= sizeof(struct in_addr);
    printf("Trace Destination: %s\n", inet_ntoa(dst));
    printf("%s\n", printCharStr(str + sizeof(struct in_addr), len));
}

void printHelloMsg(unsigned char *msg, int verbose) {
    hello_msg_t *hello = (hello_msg_t *) msg; 
    unsigned char *os = msg + sizeof(hello_msg_t);
    struct in_addr addr;
    struct in6_addr addr6;
    char v6string[SMALLBUF];

    printf(">> HELLO: ");
    if (isIPv4(hello->ip)) {
        addr.s_addr = n6ton4(hello->ip);
        printf("[Client: %s]", inet_ntoa(addr));
    } else {
        memcpy(&(addr6.s6_addr), hello->ip, IPV6ADDRLEN); 
        printf("[Client6: %s]",
          inet_ntop(AF_INET6, (struct in6_addr *)hello->ip, v6string, SMALLBUF));
    }
    printf("[Dist: %d][OSlen: %d]", hello->dist, hello->oslen);
    if (hello->oslen >0) 
        printf("[OS: %s]", printCharStr(os, hello->oslen));
    printf("[SID: %lu]\n", (unsigned long) ntohl(hello->sessionid));
}

void printReportMsg(unsigned char *msg) {
    report_msg_t *report = (report_msg_t *) msg;
    unsigned char *seq = msg + sizeof(report_msg_t);
    struct in_addr addr;
    struct in6_addr addr6;
    static char buf[SMALLBUF];

    printf("Report Message:\n");
    if (isIPv4(report->src_addr)) {
        addr.s_addr = n6ton4(report->src_addr);
        printf("\tSrc IP: %s\n", inet_ntoa(addr));
    } else {
        memcpy(addr6.s6_addr, report->src_addr, IPV6ADDRLEN);
        printf("Src: IP: %s ", inet_ntop(AF_INET6, &addr6, buf, SMALLBUF));
    }
    if (isIPv4(report->dst_addr)) {
        addr.s_addr = n6ton4(report->dst_addr);
        printf("\tDst IP: %s\n", inet_ntoa(addr));
    } else {
        memcpy(addr6.s6_addr, report->dst_addr, IPV6ADDRLEN);
        printf("Dst IP: %s ", inet_ntop(AF_INET6, &addr6, buf, SMALLBUF));
    }
    if (report->seqlen > 0) {
        printf("\tSeq(%d): ", report->seqlen);
        printf("%s\n", printCharStr(seq, report->seqlen));
    }
}

void printTraceFilter(unsigned char *msg) {
    addr_list_t *list = (addr_list_t *) msg;
    tracefilter_test_t *test = (tracefilter_test_t *) (msg + sizeof(addr_list_t));
    struct in_addr addr;
    int i;

    if (ntohs(list->size) > 100) {
        warn("bogus tracefilter list");
        return;
    }
    printf("TraceFilter (Targets: %d)\n", ntohs(list->size));
    for (i=0;i<ntohs(list->size);i++) {
        printf("\tPathLen: %d\n", test->dist);
        addr.s_addr = test->dst_ip;
        printf("\tDst IP: %s\n", inet_ntoa(addr));
        test++;
    }
}

void printMsg(unsigned char *msg, int verbose) {
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    unsigned char *payload;
    int len = ntohs(hdr->len);
    if (!verbose) {
        printf(">> %s Msg v%d: [Flag: %d][Ack: %d][Len: %d]\n",
            type(hdr->type), hdr->ver, hdr->flag, hdr->ack, len);
    } else {
        printf("Message Header:\n");
        printf("\tVer:  %d\n", hdr->ver);
        printf("\tType: %s\n", type(hdr->type));
        printf("\tFlag: %d\n", hdr->flag);
        printf("\tAck:  %d\n", hdr->ack);
        printf("\tLen:  %d\n", len);
    }
    if (len == 0) return;
    payload = msg + sizeof(spoof_msg_t);
    switch(hdr->type){
        case HELLO:
            printHelloMsg(payload, verbose);
            break;
        case PROBE_REQ:
        case PROBE6_REQ:
            printTestList(payload, verbose);
            break;
        case REPORT:
        case REPORT6:
            printReportMsg(payload);
            break;
        case TRACE:
            printTrace(payload, len);
            break;
        case TRACEFILT:
            printTraceFilter(payload);
            break;
        case ERR:
            printCharStr(payload, len);
            break;
        case DONE:
            if (verbose) printf("%s\n", printCharStr(payload, len));
            break;
        default:
            printf("*** Error, invalid message payload\n");
    }
}


/******************** message kraftwerk ********************/

int readMsg(int sock, spoof_msg_t **message, enum msg_types type) {
    unsigned char *buf = (unsigned char *) *message; 
    spoof_msg_t *msg = *message;
    int n;
    
    memset(buf, 0, BIGBUF); 
    n = Readn(sock, buf, sizeof(spoof_msg_t), CLIENT_TIMEOUT);
    if (n != sizeof(spoof_msg_t)) {
        warn("Timeout (%d) reading spoof_msg_t from socket, got %d/%d bytes",
            CLIENT_TIMEOUT, n, (int) sizeof(spoof_msg_t));
        return -1;
    }
    msg = (spoof_msg_t *) buf;
    if (ntohs(msg->len) > MAXMSGSIZE)
        return 0;
    if (ntohs(msg->len) > 0) {
        if ((n+= Readn(sock, buf + sizeof(spoof_msg_t), ntohs(msg->len), CLIENT_TIMEOUT)) <= 0)
            return 0;   /* connection closed by other end */
        if (n != sizeof(spoof_msg_t) + ntohs(msg->len)) {
            warn("Timeout (%d) reading spoof msg body from socket, got %d/%d bytes",
                CLIENT_TIMEOUT, n, (int) (sizeof(spoof_msg_t) + ntohs(msg->len)));
            return -1;
        }
#if DEBUG 
        printMsg(buf, VERBOSE);
#endif
    }
    if (msg->type != type) {
        warn("Bad message type (%d expected %d)", msg->type, type);
        return -1;
    }
    return(n);
}

void defaultMsg(spoof_msg_t *msg) {
    if (msg) {
        memset(msg, 0, sizeof(spoof_msg_t));
        msg->ver= VERSION;
        msg->type = DEFAULT;
        msg->flag = 0;
        msg->ack = 0;
        msg->len = 0;
        msg->unused = 0;
    }
}

int errorRequest(unsigned char *msg) {
    int msglen = sizeof(spoof_msg_t);
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    
    defaultMsg(hdr);
    hdr->type = ERR;
    return(msglen);
}

int helloRequest(unsigned char *msg) {
    int oslen = strlen(OS);
    int msglen = sizeof(spoof_msg_t) + sizeof(hello_msg_t) + oslen;
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    hello_msg_t *hello = (hello_msg_t *) (msg + sizeof(spoof_msg_t));
    unsigned char *os = msg + sizeof(spoof_msg_t) + sizeof(hello_msg_t);
    
    defaultMsg(hdr);
    hdr->type = HELLO;
    hdr->len = htons(sizeof(hello_msg_t) + oslen);
    memset(hello->ip, 0, IPV6ADDRLEN);
    hello->dist = 0;
    hello->unused = 0;
    hello->oslen = oslen;
    memcpy(os, OS, oslen);
    hello->sessionid = 0;
    return(msglen);
}

int reportRequest(unsigned char *msg, unsigned char *seqno, 
        unsigned char *src_addr, unsigned char *dst_addr, int nat, int ipv6) {
    int msglen = sizeof(spoof_msg_t) + sizeof(report_msg_t) + SEQNOSIZE;
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    report_msg_t *report = (report_msg_t *) (msg + sizeof(spoof_msg_t));
    unsigned char *seq = msg + sizeof(spoof_msg_t) + sizeof(report_msg_t);
    
    defaultMsg(hdr);
    if (ipv6) 
        hdr->type = REPORT6;
    else
        hdr->type = REPORT;
    hdr->len = htons(sizeof(report_msg_t) + SEQNOSIZE);
    memcpy(report->src_addr, src_addr, IPV6ADDRLEN);
    memcpy(report->dst_addr, dst_addr, IPV6ADDRLEN);
    report->seqlen = SEQNOSIZE;
    report->ack = 1;
    if (strncmp((char *)seqno, "SENDTOFAIL", strlen("SENDTOFAIL")) == 0) {
        report->ack = 0;
    }
    /* locally detected NAT interface in use; let server know */
    if (nat) {
        report->ack = 2;
    }
    memcpy(seq, seqno, SEQNOSIZE);
    return(msglen);
}

int doneRequest(unsigned char *msg) {
    int msglen = sizeof(spoof_msg_t);
    spoof_msg_t *hdr = (spoof_msg_t *) msg;

    defaultMsg(hdr);
    hdr->type = DONE;
    return(msglen);
}

int traceFilterRequest(unsigned char *msg) {
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    defaultMsg(hdr);
    hdr->type = TRACEFILT;
    return (sizeof(spoof_msg_t));
}

int probeRequest(unsigned char *msg, int v6) {
    spoof_msg_t *hdr = (spoof_msg_t *) msg;
    defaultMsg(hdr);
    if (v6) 
        hdr->type = PROBE6_REQ;
    else
        hdr->type = PROBE_REQ;
    return (sizeof(spoof_msg_t));
}

int readProbeList(unsigned char *buf, test_t **testlistptr) {
    test_t *testlist;
    addr_list_t *addr = (addr_list_t *) (buf + sizeof(spoof_msg_t));
    int numAddr = ntohs(addr->size);
    int i;

    if ((*testlistptr = calloc(numAddr, sizeof(test_t))) == NULL)
        fatal("calloc error in %s\n", __func__);
    testlist = *testlistptr;
    for (i=0;i<numAddr;i++) {
        memcpy(testlist + i, 
            buf + sizeof(spoof_msg_t) + sizeof(addr_list_t) + (sizeof(test_t)*i), 
            sizeof(test_t));
    }
    return(numAddr);
}
